/************************************************************
 *                                                          *
 * parsemid.c                                               *
 *                                                          *
 *----------------------------------------------------------*
 * Routines to play sound files via SoundBlaster DSP.       *
 *----------------------------------------------------------*
 * Copyright (c) 1994,1997 Ken Graham, All Rights Reserved  *
 *                                                          *
 * Redistribution and use in source and binary forms,       *
 * without modification, are permitted provided that the    *
 * following conditions are met:                            *
 * 1. Redistributions of source code must retain the above  *
 *    copyright notice, this list of conditions and the     *
 *    following disclaimer.                                 *
 * 2. Redistributions in binary form must reproduce the     *
 *    above copyright notice, this list of conditions and   *
 *    the following disclaimer in the documentation and/or  *
 *    other materials provided with the distribution.       *
 *                                                          *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY  *
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
 * OF SUCH DAMAGE.                                          *
 *                                                          *
 ************************************************************
 *
 * $Author:   kag  $
 * $Revision:   1.6  $
 * $Date:   19 May 1997 11:15:58  $
 *
 * History:
 *
 * $Log:   D:/snd32/midi2/archives/parsemid.c_v  $
 * 
 *    Rev 1.6   19 May 1997 11:15:58   kag
 * removed note counting code.
 * 
 *    Rev 1.5   12 May 1997 18:53:24   kag
 *  fixed "lost note off" problem
 * 
 *    Rev 1.4   11 May 1997 20:20:22   kag
 *  drums working correctly
 * 
 *    Rev 1.3   04 May 1997  0:54:12   kag
 * made changes to fix tempo timing routines
 * 
 *    Rev 1.2   02 May 1997 20:14:44   kag
 * tried to fix note count problem
 * 
 *    Rev 1.1   02 May 1997 18:50:58   kag
 *  release build for SND32 ver 2.0
 * 
 *    Rev 1.0   01 May 1996 20:40:14   kag
 * Initial revision.
 * 
 *==========================================================*/


#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <malloc.h>
#include <ctype.h>
#include "portable.h"
#include "snd32.h"
#include "snd_gm.h"

#define PROGNAME    "parsemid"

#define MIDI_NOTE_OFF       0x80
#define MIDI_NOTE_ON        0x90
#define MIDI_POLY_ATOUCH    0xA0
#define MIDI_CTRL_CHANGE    0xB0
#define MIDI_CHANGE_INST    0xC0
#define MIDI_CHNL_PRESS     0xD0
#define MIDI_PITCH_BEND     0xE0
#define MIDI_SYS_EXCLUSIVE  0xF0


#define FREADLONG(fp)  ( (fgetc(fp) << 24) + (fgetc(fp) << 16) + \
                         (fgetc(fp) << 8)  + fgetc(fp) )

#define FREADSHORT(fp) ( (fgetc(fp) << 8) + fgetc(fp) )

#define READBYTE(a) a = *song->track[track]++;

#define MAX_VOICES  18
#define OFF         0
#define ON          1

VOICE   voice[MAX_VOICES];
VOICE   waiting[MAX_TRACKS*MAX_VOICES];

/**
 ** Globals 
 **/
BYTE     tmp_byte;
LONG     timecnt = 0L;
INT      TrCount;
INT      NoteCount[MAX_TRACKS];
INT      NotePlaying = 0;
BYTE    *trackstart[32];

/**
 ** Prototypes
 **/
MIDI   *parsesong(char *filename);
MIDI   *loadMIDI(char *filename);
INT     process_event(BYTE **ptr, MIDI *song, INT track);


MIDI *loadMIDI(char *filename)
{
    int     i;
    FILE    *fpin;
    char    buffer[256];
    LONG    length;
    MIDI    *song;

    fprintf(stdout,"loadMIDI...");
    fflush(stdout);
   /*
    *  get memory for MIDI structure
    */
    if ((song = (MIDI *)malloc(sizeof(MIDI))) == NULL)
    {
        printf("malloc failed\n");
        return(NULL);
    }

    if ((fpin = fopen(filename,"rb")) == NULL)    // open midi file
    {
        printf("fopen failed can't open '%s\n", filename);
        return(NULL);
    }
                                           // Read the header

   /*
    * check for MIDI file ID
    */
    fread((void *)buffer, 1, 4, fpin);
    buffer[4] = 0;
    if (strcmp((const char *)buffer,"MThd")) 
    {
       /*
        * it's not there so bail out
        */
        printf("MIDI header check failed\n");
        fclose(fpin);
        return(NULL);
    }

   /*
    * get length of MIDI header chunk (should be 6)
    */
    length  = FREADLONG(fpin);

   /*
    * get MIDI Format specifier
    */
    song->format = FREADSHORT(fpin);
   /*
    * check for MIDI Type 0 (all info in 1 track) or
    * MIDI Type 1 (multiple tracks) - bail out on anything else
    */
    if (song->format != 0 && song->format != 1) 
    {
        printf("song not format 0 or 1\n");
        free(song);
        fclose(fpin);
        return(NULL);
    }

   /*
    * get number of tracks
    */
    song->tracks = FREADSHORT(fpin);
   /*
    * check for a valid number of tracks
    */
    if (song->tracks < 1 || song->tracks > 32) 
    {
        printf("tracks > 32\n");
        free(song);
        fclose(fpin);
        return(NULL);
    }

   /*
    * get time division
    */
    song->tdivision  = FREADSHORT(fpin);
    if (song->tdivision < 0)
    {
        song->tdivision = -song->tdivision;
    }
    if (song->tdivision == 0)
    {
        song->tdivision = 96;
    }

   /*
    * check if at end of MID header - seek if we're not
    */
    if (length != 6)
    {
        fseek(fpin, length - 6, SEEK_CUR);
    }
   /*
    * clear track pointers
    */
    for (i=0;i<MAX_TRACKS;i++)
    {
        song->track[i] = trackstart[i] = NULL;
    }

   /*
    * Read individual track data
    */
    for (i = 0; i < song->tracks; i++) 
    {
       /*
        * get midi track id
        */
        fread((void *)buffer, 1, 4, fpin);
        buffer[4] = 0;
        if (strcmp((const char *)buffer,"MTrk")) 
        {
            printf("MTrk check failed\n");
            free(song);
            fclose(fpin);
            return(NULL);
        }

       /*
        * length of MIDI track (exclusive of track header)
        */
        length = FREADLONG(fpin);

       /*
        * get memory for track
        */
        if ((song->track[i] = (BYTE *)malloc(length)) == NULL)
        {
            printf("track malloc failed\n");
            while (i) 
            {
                i--;
                free(song->track[i]);
            }
            free(song);
            fclose(fpin);
            return(NULL);
        }
       /*
        * read track into memory
        */
        if (fread(song->track[i], 1, (size_t)length, fpin) != length) 
        {
            while (i) 
            {
                i--;
                free(song->track[i]);
            }
            printf("track read failed\n");
            free(song);
            fclose(fpin);
            return(NULL);
        }
        trackstart[i] = song->track[i];
    }

    fclose(fpin);
    fprintf(stdout,"done\n");
    fflush(stdout);
    return(song);
}

/*
 * seek to the midi stream and handle midi events for the song
 */
MIDI *parsesong(char *filename)
{
    INT     i;
    INT     delta    = 0;
    INT     done     = 0;
    MIDI   *song;

    if ((song = loadMIDI(filename)) == NULL)
    {
        printf("loadMIDI failed\n");
        return(NULL);
    }

    printf("Midi Format:        %d\n", song->format);
    printf("Number of Tracks:   %d\n", song->tracks);

   /*
    * setup midi variables
    */
    TrCount = song->tracks;
    for (i=0;i<song->tracks;i++)
    {
        song->timebase[i]  = 0L;            /* start at time 0          */
        song->lastevent[i] = (BYTE)MIDI_NOTE_OFF; /* set last event     */
        song->active[i]    = 1;             /* set track to active      */
        song->track[i]++;                   /* skip first time value    */
    }

    done = 0;
    while (!done)
    {
       /* 
        * process tracks
        */
        for (i=0;i<song->tracks;i++) 
        {
           /*
            * check if track is active and skip if it is not
            */
            if (song->active[i] == 0)
            {
                continue;
            }
           /*
            * get MIDI events while time is now
            */
            while (song->timebase[i] <= 0)
            {
               /*
                * process midi events
                */
                if (process_event(&song->track[i], song, i) != 0) 
                {
                    return(song);
                }
                if (song->active[i])
                {
                   /*
                    * get delta time 
                    */
                    {
                        INT     value;
                        BYTE    tmp_byte;

                        tmp_byte = *song->track[i]++;
                        value = (int)tmp_byte;
                        if (tmp_byte & 0x80) 
                        {
                            value &= 0x7F;
                            do 
                            {
                                tmp_byte = *song->track[i]++;
                               /*
                                * only lowest 7 bits are used
                                */
                                value = (value * 128) + (tmp_byte & 0x7F);
                            } while (tmp_byte & 0x80);
                        }
                        delta = value;
                    }
                    song->timebase[i] += delta;
                }
            }
            song->timebase[i]--;
        }
        timecnt++;
    }
    return(song);
}

INT get_free_voice(void)
{
    INT     i;

    for (i=0;i<MAX_VOICES;i++) 
    {
        if (voice[i].on == OFF) 
        {
            return(i);
        }
    }
    return(-1);
}

INT find_voice(BYTE channel, INT note)
{
    INT     i;

    for (i=0;i<MAX_VOICES;i++) 
    {
        if (voice[i].on == ON) 
        {
            if (voice[i].chan == channel && voice[i].freq == note) 
            {
                return(i);
            }
        }
    }
    return(-1);
}

INT Add_To_Waiting(BYTE chan, INT note, INT track)
{
    int     i;
    for (i=0;i<MAX_TRACKS*MAX_VOICES;i++)
    {
        if (waiting[i].on == OFF)
        {
            waiting[i].on    = ON;
            waiting[i].chan  = chan;
            waiting[i].freq  = note;
            waiting[i].track = track;
            return(i);
        }
    }
    return(-1);
}

INT Kill_On_Waiting(BYTE chan, INT note, INT track)
{
    int     i;
    for (i=0;i<MAX_TRACKS*MAX_VOICES;i++)
    {
        if (waiting[i].on == ON)
        {
            if (waiting[i].chan == chan  && 
                waiting[i].freq == note  &&
                waiting[i].chan == chan) 
            {
                waiting[i].on    = OFF;
                return(i);
            }
        }
    }
    return(-1);
}

/*
 * process a midi event
 */
INT process_event(BYTE **ptr, MIDI *song, int track)
{
    INT         count;
    INT         i;
    INT         sbvoice;
    INT         trk = track+1;  // Use 1 based not 0 based track numbering
    BYTE        channel;
    BYTE        note;
    BYTE        volume;
    ULONG        tempo;
    static INT  status = -1;

   /* 
    * get status byte 
    */
    READBYTE(tmp_byte);
    if (tmp_byte & 0x80) 
    {
        status = (INT)tmp_byte;
        printf(" ");
    }
    else                                /* a running event  */
    {
        status = song->lastevent[track];/* use last event   */
        (*ptr)--;                       /* back up 1 char   */
        printf("<");
    }
    channel = (BYTE)(status & 0x0F);
   /*
    * switch different events 
    */
    printf("%08d: tr:%02d ", timecnt, trk);
    switch (status & 0xF0) 
    {
    case MIDI_NOTE_OFF:         /* turn note off    */
        NoteCount[trk]--;        // delete note from running count
       /*
        * read MIDI note & velocity
        */
        READBYTE(note);
        READBYTE(volume);         /* velocity */

        sbvoice = find_voice(channel, note);
        if (sbvoice != -1)
        {
            voice[sbvoice].on   = OFF;
            printf("NOTE OFF ch:%02d note:%02d\n", 
                channel, note);
            NotePlaying--;
        }
        else
        {
            if (Kill_On_Waiting(channel, note, track) == -1)
            {
                printf("NOTE OFF ch:%02d note:%02d\n", 
                    channel, note);
            }
            else
            {
                printf("NOTE OFF ch:%02d note:%02d\n", 
                    channel, note);
                NotePlaying--;
            }
        }
        fflush(stdout);
        break;
    case MIDI_NOTE_ON:          /* turn note on     */
       /* 
        * get midi note 
        */
        READBYTE(note);
       /*
        * read Velocity (Volume) value
        */
        READBYTE(tmp_byte);
        if (tmp_byte == 0) 
        {
            NoteCount[trk]--;        // delete note from running count
            sbvoice = find_voice(channel, note);
            if (sbvoice != -1)
            {
                voice[sbvoice].on   = OFF;
                printf("NOTE OFF ch:%02d note:%02d\n", 
                    channel, note);
                NotePlaying--;
            }
            else
            {
                if (Kill_On_Waiting(channel, note, track) == -1)
                {
                    printf("NOTE OFF ch:%02d note:%02d\n", 
                        channel, note);
                }
                else
                {
                    printf("NOTE OFF ch:%02d note:%02d\n", 
                        channel, note);
                    NotePlaying--;
                }
            }
            fflush(stdout);
        }
       /*
        * turn note on 
        */
        else 
        {
            NoteCount[trk]++;      // add to note running count

            volume = tmp_byte;
            sbvoice = get_free_voice();
            if (sbvoice != -1)
            {
                voice[sbvoice].on    = ON;
                voice[sbvoice].chan  = channel;
                voice[sbvoice].freq  = note;
                voice[sbvoice].track = track;
                if (voice[sbvoice].inst != song->instrument[channel])
                {
                    voice[sbvoice].inst = song->instrument[channel];
                }
                printf("NOTE ON  ch:%02d note:%02d\n", 
                    channel, note);
                NotePlaying++;
            }
            else
            {
                if (Add_To_Waiting(channel, note, track) == -1)
                {
                    printf("NOTE ON  ch:%02d note:%02d\n", 
                        channel, note);
                }
                else
                {
                    printf("NOTE ON  ch:%02d note:%02d\n", 
                        channel, note);
                    NotePlaying++;
                }
            }
            fflush(stdout);
        }
        break;
    case MIDI_CHANGE_INST:      /* Change Instrument        */
       /*
        * change the instrument on a channel 
        */
        READBYTE(tmp_byte);
        printf("LOAD INST   ch:%02d inst:%03d (%s)\n", 
            channel, tmp_byte, gm_inst[tmp_byte]);
        fflush(stdout);
        song->instrument[channel] = tmp_byte;
        break;
    case MIDI_CHNL_PRESS:       /* Channel Pressure         */
       /*
        * read pressure value & toss it
        */
        READBYTE(tmp_byte);
        printf("CHNL PRESS ch:%02d press:%03d \n", channel, tmp_byte);
        fflush(stdout);
        break;
    case MIDI_POLY_ATOUCH:      /* PolyPhonic Aftertouch    */
       /*
        * read pitch bend values & toss 'em
        */
        READBYTE(tmp_byte);
        READBYTE(tmp_byte);
        printf("POLY AFTERTOUCH ch:%02d \n", channel);
        fflush(stdout);
        break;
    case MIDI_CTRL_CHANGE:      /* Control Change           */
       /*
        * read pitch bend values & toss 'em
        */
        {
            BYTE    controller;
            BYTE    value;
    
            READBYTE(controller);
            READBYTE(value);
            printf("CTRL CHANGE ");
            switch(controller)
            {
            case 0:
                printf("Bank Select       value:%03d ", value);
                break;
            case 1:
                printf("Modulation Wheel  value:%03d ", value);
                break;
            case 2:
                printf("Breath Controller value:%03d ", value);
                break;
            case 3:
                printf("Undefined         value:%03d ", value);
                break;
            case 4:
                printf("Foot Controller   value:%03d ", value);
                break;
            case 5:
                printf("Portamento time   value:%03d ", value);
                break;
            case 6:
                printf("Data entry MSB    value:%03d ", value);
                break;
            case 7:
                printf("Main volume       value:%03d ", value);
                break;
            case 64:
                printf("Sustain           value:%03d ", value);
                break;
            case 65:
                printf("Portamento        value:%03d ", value);
                break;
            case 66:
                printf("Sostenuto Pedal   value:%03d ", value);
                break;
            case 67:
                printf("Soft Pedal        value:%03d ", value);
                break;
            case 68:
                printf("Data increment    value:%03d ", value);
                break;
            case 69:
                printf("Data decrement    value:%03d ", value);
                break;
            case 122:
                if (value == 127)
                {
                    printf("LOCAL ON ");
                }
                else if (value == 0)
                {
                    printf("LOCAL OFF");
                }
                break;
            case 123:
                printf("ALL NOTES OFF ");
                for (i=0;i<MAX_TRACKS;i++)
                {
                    NoteCount[i] = 0;
                }
                for (i=0;i<MAX_VOICES;i++)
                {
                    voice[i].on = OFF;
                }
                break;
            case 124:
                printf("OMNI OFF ");
                break;
            case 125:
                printf("OMNI ON ");
                break;
            case 126:
                printf("MONO ON ");
                break;
            case 127:
                printf("POLY ON ");
                break;
            default:
                printf("ch:%02d contr:%03d   value:%03d ", 
                    channel, controller, value);
                break;
            }
            printf("\n");
            fflush(stdout);
        }
        break;
    case MIDI_PITCH_BEND:       /* Pitch Bend               */
       /*
        * read pitch bend values & toss 'em
        */
        READBYTE(tmp_byte);
        READBYTE(tmp_byte);
        printf("PITCH BEND ch:%02d \n", channel);
        break;
    case MIDI_SYS_EXCLUSIVE:    /* System Exclusive     */
        switch (status) 
        {
            case 240:   /* System Exclude message   */
               /*
                * get the system manufacturer byte
                */
                READBYTE(tmp_byte);
               /*
                * read data until End of Exclusive byte
                */
                printf("SYSEX: bytes: ");
                while (tmp_byte != 247) 
                {
                    READBYTE(tmp_byte);
                    printf("0x%02x ", (UCHAR)tmp_byte);
                }
                printf("\n");
                break;
            case 242:   /* Song Position Pointer    */
               /*
                * read song position & toss it
                */
                printf("SONG POSN \n");
                READBYTE(tmp_byte);
                READBYTE(tmp_byte);
                break;
            case 243:   /* Song Select message      */
               /*
                * read number & toss it
                */
                printf("SONG SELCT \n");
                READBYTE(tmp_byte);
                break;
            case 246:   /* Tune Request message     */
                printf("TUNE REQUEST \n");
                break;
            case 255:   /* System Reset message or meta event     */
                READBYTE(tmp_byte);
                switch (tmp_byte) 
                {
                case 0x01:          /* text event           */
                    printf("TEXT EVENT: '");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("%c", tmp_byte);
                    }
                    printf("'\n");
                    break;
                case 0x02:          /* copyright notice     */
                    printf("COPYRIGHT: '");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("%c", tmp_byte);
                    }
                    printf("'\n");
                    break;
                case 0x03:          /* sequence/track name  */
                    printf("TRACK NAME: '");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("%c", tmp_byte);
                    }
                    printf("'\n");
                    break;
                case 0x04:          /* instrument name      */
                    printf("INSTRUMENT: '");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("%c", tmp_byte);
                    }
                    printf("'\n");
                    break;
                case 0x05:          /* lyric                */
                    printf("LYRIC: '");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("%c", tmp_byte);
                    }
                    printf("'\n");
                    break;
                case 0x06:          /* Marking                */
                    printf("MARKING: '");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("%c", tmp_byte);
                    }
                    printf("'\n");
                    break;
                case 0x07:          /* Cue Point                */
                    printf("CUE POINT: '");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("%c", tmp_byte);
                    }
                    printf("'\n");
                    break;
                case 0x20:          /* Channel Prefix           */
                    printf("CHANNEL PREFIX: ");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("0x%02x ", tmp_byte);
                    }
                    printf("\n");
                    break;
                case 0x21:          /* Channel Selector         */
                    printf("CHANNEL SELECTOR: ");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("0x%02x ", tmp_byte);
                    }
                    printf("\n");
                    break;
                case 0x54:          /* SMPTE                */
                    printf("SMPTE: ");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("0x%02x ", tmp_byte);
                    }
                    printf("\n");
                    break;
                case 0x58:          /* Time Sig             */
                    printf("TIME SIG: ");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("0x%02x ", tmp_byte);
                    }
                    printf("\n");
                    break;
                case 0x59:          /* Key Sig              */
                    printf("KEY SIG: ");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("0x%02x ", tmp_byte);
                    }
                    printf("\n");
                    break;
                case 0x51:          /* set tempo            */
                    tempo = 0;
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        tempo *= 256;
                        tempo += (tmp_byte & 0xff);
                    }
                    printf("SET TEMPO: tempo = %d\n", tempo);
                    break;
                case 0x2F:          /* end of track */
                   /*
                    * decrement number of tracks
                    */
                    TrCount--;
                   /*
                    * mark track as not active
                    */
                    printf("TRACK END: [left: %02d]\n", TrCount);
                    song->active[track]   = 0;
                    song->timebase[track] = 10000;
                   /*
                    * check if we are done
                    */
                    if (TrCount == 0)
                    {
                        printf("End of Song\n");
                        return(-1);
                    }
                    break;
                case 0x7f:          /* Sequencer event  */
                    printf("SEQUENCER: ");
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("0x%02x ", tmp_byte);
                    }
                    printf("\n");
                    break;

                default:
                    printf("UNKNOWN META EVENT 0x%02x: ", tmp_byte);
                    READBYTE(count);
                    for (i=0;i<count;i++)   /* read past the field  */
                    {
                        READBYTE(tmp_byte);
                        printf("0x%02x ", tmp_byte);
                    }
                    printf("\n");
                    break;
                }
                break;
           /*
            * Real Time Messages
            */
            case 248:   /* Timing Clock Message     */
                printf("RTM: Timing Clock Message \n");
                break;
            case 250:   /* Start message            */
                printf("RTM: Start message \n");
                break;
            case 251:   /* Continue message         */
                printf("RTM: Continue message \n"); 
                break;
            case 252:   /* Stop message             */
                printf("RTM: Stop message \n");
                break;
            case 254:   /* Active Sensing message   */
                printf("RTM: Active Sensing message \n");
                break;
            default:
                break;
        }
        break;
    default:
        printf("Internal program error\n");
        exit(-1);
    }
    song->lastevent[track] = status;
    return(0);
}


/*
 * Main Program Loop
 */
VOID main(INT argc, CHAR *argv[])
{
    INT     i;
    CHAR    ifilename[64];
    MIDI   *song;

    fprintf(stderr, "%s: Copyright (c) 1994,1997 Ken Graham\n", PROGNAME);
    if (argc < 2) 
    {
        fprintf(stderr,"%s: Syntax: %s file[.mid] \n", PROGNAME, PROGNAME);
        fprintf(stderr,"  Input file is a .MID file (file extension is optional if .MID)\n");
        exit(-1);
    }

   /*
    * initialize stuff
    */
    ifilename[0] = '\0';
    memset(voice,   OFF, (sizeof(VOICE)*MAX_VOICES));
    memset(waiting, OFF, (sizeof(VOICE)*MAX_VOICES*MAX_TRACKS));
   /*
    * scan for option flags
    */
    for (i=1;i<argc;i++) 
    {
        if (argv[i][0] == '/' || argv[i][0] == '-') 
        {
            switch(toupper(argv[i][1])) 
            {
            case '?':
                fprintf(stderr,"%s: Syntax: %s file[.mid] \n", PROGNAME, PROGNAME);
                fprintf(stderr,"  Input file is a .MID file (file extension is optional if .MID)\n");
                exit(-1);
                
            default:
                fprintf(stderr, "%s: Unknown Parameter: '%s'\n", PROGNAME, argv[i]);
                exit(1);
            }
        }
        else 
        {
            strlwr(argv[i]);
            if (strlen((const char *)ifilename) == 0) 
            {
                CHAR    *ptr;
                
                strcpy((char *)ifilename, argv[i]);
                if ((ptr = strchr(argv[i], '.')) == NULL) 
                {
                    strcat((char *)ifilename, ".mid");
                }
            }
        }
    }
    for (i=0;i<MAX_TRACKS+1;i++)
    {
        NoteCount[i] = 0;
    }
   /*
    * Parse Song
    */
    if ((song = parsesong((char *)ifilename)) == NULL)
    {
        printf("parsesong failed\n");
        exit(-1);
    }
    for (i=0;i<MAX_TRACKS+1;i++)
    {
        printf("NoteCount[%d] = %d\n", i, NoteCount[i]);
    }

   /*
    * free memory
    */
    for (i=0;i<song->tracks;i++)
    {
        free(trackstart[i]);
    }
    free(song);

    exit(0);
}


