/************************************************************
 *                                                          *
 * playsnd.c                                                *
 *                                                          *
 *----------------------------------------------------------*
 * Routines to play sound files via SoundBlaster DSP.       *
 *----------------------------------------------------------*
 * Copyright (c) 1994 Ken Graham, All Rights Reserved       *
 *                                                          *
 * Redistribution and use in source and binary forms,       *
 * without modification, are permitted provided that the    *
 * following conditions are met:                            *
 * 1. Redistributions of source code must retain the above  *
 *    copyright notice, this list of conditions and the     *
 *    following disclaimer.                                 *
 * 2. Redistributions in binary form must reproduce the     *
 *    above copyright notice, this list of conditions and   *
 *    the following disclaimer in the documentation and/or  *
 *    other materials provided with the distribution.       *
 *                                                          *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY  *
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
 * OF SUCH DAMAGE.                                          *
 *                                                          *
 ************************************************************
 *
 * $Author:   kag  $
 * $Revision:   1.2  $
 * $Date:   02 May 1997 19:53:32  $
 *
 * History:
 *
 * $Log:   D:/snd32/midi2/archives/playsnd.c_v  $
 * 
 *    Rev 1.2   02 May 1997 19:53:32   kag
 *  cleaned up loose ends regarding exposing SB parameters
 * 
 *    Rev 1.1   02 May 1997 18:51:06   kag
 *  release build for SND32 ver 2.0
 * 
 *    Rev 1.0   07 May 1996  0:10:06   kag
 * Initial revision.
 * 
 *    Rev 1.1   23 Nov 1994  1:23:04   kag
 * removed hard coded value reset
 * 
 *    Rev 1.0   15 Nov 1994 21:48:52   kag
 * Initial revision.
 *
 ************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <conio.h>
#include <ctype.h>
#include "portable.h"
//#include "sndext.h"
#include "snd32.h"

#define PROGNAME    "playsnd"

char spinchar[] = { "|\\-/" };

void main(int argc, char *argv[])
{
    DIGSOUND     ds;
    INT          i;
    INT          chan;
    INT          cardtype;
    INT          count = 0;
    INT          verbose = FALSE;
    time_t       StartTime;
    time_t       EndTime;

    fprintf(stderr, "%s: Copyright (c) 1994,1996 Ken Graham. All Rights Reserved.\n", PROGNAME);
    fprintf(stderr, "SND32 Routines %s\n", CopyRightStr);
    if (argc < 2) 
    {
        fprintf(stderr,"%s: Usage: %s [/v] sndfile.[WAV | RAW]\n",
            PROGNAME, PROGNAME);
        fprintf(stderr, "  /v sets verbose mode\n");
        exit(1);
    }
   /*
    * Test for sound card type
    */
    cardtype = SndCardType();
    if (cardtype == NOCARD) 
    {
        fprintf(stderr, "%s: sound card not found.\n", PROGNAME);;
        exit(1);
    }
    for (i=1;i<argc;i++)
    {
        if (argv[i][0] == '-' || argv[i][0] == '/')
        {
            switch (toupper(argv[i][1]))
            {
            case 'V':
                verbose = TRUE;
                break;
            default:
                printf("%s: Error: unknown parameter '%s' ignored\n", 
                    PROGNAME, argv[i]);
                break;
            }
        }
        else
        {
            ds.name = argv[i];
        }
    }
    if (verbose == TRUE)
    {
        PrintSB(stdout);
    }

    if (DSPload((DIGSOUND *)&ds) == NULL) 
    {
        fprintf(stderr, "%s: Error: Unable to load sound file '%s'\n", 
            PROGNAME, ds.name);
        exit(1);
    }
    else 
    {
        printf("ds.len  = %d (0x%04x)\n", ds.len, ds.len);
        printf("ds.data = 0x%08lx\n", ds.data);
        printf("Press a key to start sound\n");
        getch();
        StartTime = time(NULL);
        chan = DSPplay((DIGSOUND *)&ds, -1);
    }
   /*
    * Play sound in a loop - until user presses ESC
    */
    printf("Sound should be playing. Press ESC to exit.\n");
    printf("  Playing");
    while (DSPplaying()) 
    {
        /* 
         * need to eat chars - DOS beep screws up irqs
         */
        if (kbhit()) 
        {
            INT key;
            /*
             * check for ESC
             */
            if ((key = getch()) == 0x1b) 
            {
                DSPstop(chan);
                break;
            }
        }
        printf("\r%c", spinchar[++count&3]);
    }

    DSPcleanup();

    EndTime = time(NULL);
    printf("\n\nStart Time: %s", ctime((time_t *)&StartTime));
    printf("End Time  : %s", ctime((time_t*)&EndTime));
    printf("Elapsed time:  %f seconds\n", difftime(EndTime, StartTime));
    
    exit(0);
}










