/************************************************************
 *                                                          *
 * snd32.h                                                  *
 *                                                          *
 *----------------------------------------------------------*
 * Defines, structures, prototypes and variables used by    *
 * the SND32 Library routines to play sound files.          *
 *----------------------------------------------------------*
 * Copyright (c) 1994,1996 Ken Graham, All Rights Reserved  *
 *----------------------------------------------------------*
 *                                                          *
 * Redistribution and use in source and binary forms,       *
 * without modification, are permitted provided that the    *
 * following conditions are met:                            *
 * 1. Redistributions of source code must retain the above  *
 *    copyright notice, this list of conditions and the     *
 *    following disclaimer.                                 *
 * 2. Redistributions in binary form must reproduce the     *
 *    above copyright notice, this list of conditions and   *
 *    the following disclaimer in the documentation and/or  *
 *    other materials provided with the distribution,       *
 *    unless permission is obtained from the author in      *
 *    writing.                                              *
 *                                                          *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY  *
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT        *
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY    *
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.     *
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,   *
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR             *
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,    *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF     *
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER *
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN        *
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING           *
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE   *
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY *
 * OF SUCH DAMAGE.                                          *
 *                                                          *
 *----------------------------------------------------------*
 * SoundBlaster and SoundBlaster Pro are trademarks of      *
 * Creative Labs, Inc.                                      *
 * Other trademarks are the property of their respective    *
 * owners.                                                  *
 ************************************************************
 *
 * $Author:   kag  $
 * $Revision:   1.6  $
 * $Date:   13 May 1997 22:48:00  $
 *
 * History:
 *
 * $Log:   D:/snd32/midi2/archives/snd32.h_v  $
   
      Rev 1.6   13 May 1997 22:48:00   kag
   added api for mixer and removed MIDI event defines
   
      Rev 1.5   12 May 1997 18:53:00   kag
    fixed "lost note off" problem
   
      Rev 1.4   11 May 1997 20:19:28   kag
   drums working correctly
   
      Rev 1.3   04 May 1997  0:53:18   kag
   made changes to fix tempo timing routines
   
      Rev 1.2   02 May 1997 19:54:02   kag
    cleaned up loose ends regarding exposing SB parameters
   
      Rev 1.1   02 May 1997 18:38:54   kag
    release build for SND32 ver 2.0
   
      Rev 1.0   18 May 1996 16:42:18   kag
   Initial revision.
   
      Rev 1.0   15 Nov 1994 20:55:46   kag
   Initial revision.
 *
 ************************************************************/

#if !defined(_SND_32_H_)
#define _SND_32_H_

#include <stdio.h>

#if !defined(_PORTABLE_H_)
#include "portable.h"
#endif

/*----------------------------------------------------------*
 * Test for Sound Card Type                                 *
 *----------------------------------------------------------*/
INT     SndCardType(VOID);          /* test for card parms  */

/*----------------------------------------------------------*
 * Sound Card Types                                         *
 *----------------------------------------------------------*/
#define SB16    5       /* uses 1 Yamaha OPL3 sound chip    */
#define SBPRO2  4       /* uses 1 Yamaha OPL3 sound chip    */
#define SBPRO   3       /* uses 2 Yamaha OPL2 sound chips   */
#define SB      2       /* uses 1 Yamaha OPL2 sound chip    */
#define ADLIB   1       /* uses 1 Yamaha OPL2 sound chip    */
#define NOCARD  0

#define SUCCESS 1
#define FAILURE 0

/**========================================================**
 **                                                        **
 ** DIGITAL Sound Typedefs, Prototypes and Defines         **
 **                                                        **
 **========================================================**/

/*----------------------------------------------------------*
 * Digital Sound basic structure                            *
 *----------------------------------------------------------*/
typedef struct {
    CHAR *name;
    BYTE *data;
    INT   len;
    INT   rate;
} DIGSOUND;

/*----------------------------------------------------------*
 * Digital Sound PUBLIC Routines                            *
 *----------------------------------------------------------*/
INT   DSPinit(VOID);
BYTE *DSPload(DIGSOUND *ds);
INT   DSPplay(DIGSOUND *ds, INT chan);
INT   DSPplaying(VOID);
VOID  DSPstop(INT chan);
VOID  DSPcleanup(VOID);

/**========================================================**
 **                                                        **
 ** MIDI Music Typedefs, Prototypes and Defines            **
 **                                                        **
 **========================================================**/

/*----------------------------------------------------------*
 * the maximum number of MIDI Tracks supported              *
 *----------------------------------------------------------*/
#define MAX_TRACKS  32

/*----------------------------------------------------------*
 * the maximum number of voices available on an OPL3 based  *
 * sound card                                               *
 *----------------------------------------------------------*/
#define MAX_VOICES  18
#define MAX_PERC     5

/*----------------------------------------------------------*
 * MIDI Structures                                          *
 *----------------------------------------------------------*/
typedef struct 
{
    SHORT   format;                 /* MIDI format 0 & 1 supported  */
    SHORT   tracks;                 /* number of tracks             */
    SHORT   tdivision;              /* song time division           */
    SHORT   loop;                   /* number of times to loop      */
    UINT    tempo;                  /* song tempo                   */
    UINT    active[MAX_TRACKS];     /* track active flag            */
    UINT    instrument[MAX_TRACKS]; /* current instruments          */
    UINT    tracklen[MAX_TRACKS];   /* track lengths                */
    LONG    timebase[MAX_TRACKS];   /* current timebase for tracks  */
    BYTE   *track[MAX_TRACKS];      /* pointer to track data        */
    BYTE    lastevent[MAX_TRACKS];  /* last MIDI event              */

} MIDI;

/*
 * OPL2/3 Voice Structure
 */
typedef struct 
{
    UINT     on;
    UINT     chan;
    UINT     freq;
    UINT     inst;
    UINT     track;
    UINT     count;
} VOICE;

#define OFF         0
#define ON          1

/*----------------------------------------------------------*
 * MIDI Control API                                         *
 *----------------------------------------------------------*/
MIDI *MIDI_Load(char *filename);
VOID  MIDI_Play(MIDI *SongData, int loop);
INT   MIDI_Playing(VOID);
VOID  MIDI_Stop(VOID);
VOID  MIDI_Free(MIDI *song);


/*----------------------------------------------------------*
 * Common Utility Public Routines                           *
 *----------------------------------------------------------*/
VOID PrintSB(FILE *fp);
extern const BYTE *CopyRightStr;

/*----------------------------------------------------------*
 * Basic Mixer Control API                                  *
 *----------------------------------------------------------*/

/*----------------------------------------------------------*
 * Mixer registers (on SBPro & up)                          *
 *----------------------------------------------------------*/
#define VOC_VOL      0x04       /* DSP output volume        */
#define MIC_VOL      0x0A       /* Microphone output vol    */
#define MASTER_VOL   0x22       /* Master sound volume      */
#define FM_VOL       0x26       /* FM output volume         */
#define CD_VOL       0x28       /* CD output volume         */
#define LINE_VOL     0x2E       /* Line output volume       */

VOID SetMixerVolume (INT volume, BYTE what);

#endif
